@@ -4,6 +4,8 @@
  * (c) 2001 Gerd Knorr <kraxel@bytesex.org>
  *
  */
+
+#define NG_PRIVATE
 #include "config.h"
 
 #include <stdio.h>
@@ -15,10 +17,9 @@
 #include <fnmatch.h>
 #include <errno.h>
 #include <ctype.h>
+#include <inttypes.h>
 #include <sys/time.h>
-#ifdef HAVE_ENDIAN_H
-# include <endian.h>
-#endif
+#include <sys/types.h>
 
 #include <dlfcn.h>
 #ifndef RTLD_NOW
@@ -51,11 +52,12 @@ const unsigned int ng_vfmt_to_depth[] = {
     32,              /* RGB32    */
     16,              /* LUT2     */
     32,              /* LUT4     */
-    16,		     /* YUV422   */
+    16,		     /* YUYV     */
     16,		     /* YUV422P  */
     12,		     /* YUV420P  */
     0,		     /* MJPEG    */
     0,		     /* JPEG     */
+    16,		     /* UYVY     */
 };
 
 const char* ng_vfmt_to_desc[] = {
@@ -72,20 +74,21 @@ const char* ng_vfmt_to_desc[] = {
     "32 bit TrueColor (BE: -rgb)",
     "16 bit TrueColor (lut)",
     "32 bit TrueColor (lut)",
-    "16 bit YUV 4:2:2 (packed)",
+    "16 bit YUV 4:2:2 (packed, YUYV)",
     "16 bit YUV 4:2:2 (planar)",
     "12 bit YUV 4:2:0 (planar)",
     "MJPEG (AVI)",
     "JPEG (JFIF)",
+    "16 bit YUV 4:2:2 (packed, UYVY)",
 };
 
 /* --------------------------------------------------------------------- */
 
 const unsigned int   ng_afmt_to_channels[] = {
-    0,  1,  2,  1,  2,  1,  2
+    0,  1,  2,  1,  2,  1,  2, 0
 };
 const unsigned int   ng_afmt_to_bits[] = {
-    0,  8,  8, 16, 16, 16, 16
+    0,  8,  8, 16, 16, 16, 16, 0
 };
 const char* ng_afmt_to_desc[] = {
     "none",
@@ -94,7 +97,8 @@ const char* ng_afmt_to_desc[] = {
     "16bit mono (LE)",
     "16bit stereo (LE)",
     "16bit mono (BE)",
-    "16bit stereo (BE)"
+    "16bit stereo (BE)",
+    "mp3 compressed audio",
 };
 
 /* --------------------------------------------------------------------- */
@@ -175,9 +179,26 @@ ng_malloc_video_buf(struct ng_video_fmt *fmt, int size)
 
 /* --------------------------------------------------------------------- */
 
+struct ng_audio_buf*
+ng_malloc_audio_buf(struct ng_audio_fmt *fmt, int size)
+{
+    struct ng_audio_buf *buf;
+
+    buf = malloc(sizeof(*buf)+size);
+    memset(buf,0,sizeof(*buf));
+    buf->fmt  = *fmt;
+    buf->size = size;
+    buf->data = (char*)buf + sizeof(*buf);
+    return buf;
+}
+
+/* --------------------------------------------------------------------- */
+
 struct ng_attribute*
 ng_attr_byid(struct ng_attribute *attrs, int id)
 {
+    if (NULL == attrs)
+	return NULL;
     for (;;) {
 	if (NULL == attrs->name)
 	    return NULL;
@@ -190,6 +211,8 @@ ng_attr_byid(struct ng_attribute *attrs, int id)
 struct ng_attribute*
 ng_attr_byname(struct ng_attribute *attrs, char *name)
 {
+    if (NULL == attrs)
+	return NULL;
     for (;;) {
 	if (NULL == attrs->name)
 	    return NULL;
@@ -225,9 +248,10 @@ ng_attr_getint(struct ng_attribute *attr, char *value)
     if (attr->type != ATTR_TYPE_CHOICE)
 	return -1;
 
-    for (i = 0; attr->choices[i].str != NULL; i++)
+    for (i = 0; attr->choices[i].str != NULL; i++) {
 	if (0 == strcasecmp(attr->choices[i].str,value))
 	    return attr->choices[i].nr;
+    }
 
     if (isdigit(value[0])) {
 	/* Hmm.  String not found, but starts with a digit.
@@ -344,24 +368,14 @@ ng_ratio_fixup2(int *width, int *height, int *xoff, int *yoff,
 
 /* --------------------------------------------------------------------- */
 
-struct ng_video_conv  **ng_conv;
-struct ng_filter      **ng_filters;
-struct ng_writer      **ng_writers;
-struct ng_vid_driver  **ng_vid_drivers;
-struct ng_dsp_driver  **ng_dsp_drivers;
-struct ng_mix_driver  **ng_mix_drivers;
-
-static void ng_register_listadd(void ***list, void *add)
-{
-    int n = 0;
-
-    if (*list)
-	for (n = 0; NULL != (*list)[n]; n++)
-	    /* nothing */;
-    *list = realloc(*list,sizeof(void*)*(n+2));
-    (*list)[n++] = add;
-    (*list)[n++] = NULL;
-}
+LIST_HEAD(ng_conv);
+LIST_HEAD(ng_aconv);
+LIST_HEAD(ng_filters);
+LIST_HEAD(ng_writers);
+LIST_HEAD(ng_readers);
+LIST_HEAD(ng_vid_drivers);
+LIST_HEAD(ng_dsp_drivers);
+LIST_HEAD(ng_mix_drivers);
 
 static int ng_check_magic(int magic, char *plugname, char *type)
 {
@@ -383,10 +397,23 @@ ng_conv_register(int magic, char *plugname,
 {
     int n;
 
-    if (0 != ng_check_magic(magic,plugname,"converters"))
+    if (0 != ng_check_magic(magic,plugname,"video converters"))
 	return -1;
     for (n = 0; n < count; n++)
-	ng_register_listadd((void***)(&ng_conv),&list[n]);
+	list_add_tail(&(list[n].list),&ng_conv);
+    return 0;
+}
+
+int
+ng_aconv_register(int magic, char *plugname,
+		  struct ng_audio_conv *list, int count)
+{
+    int n;
+    
+    if (0 != ng_check_magic(magic,plugname,"audio converters"))
+	return -1;
+    for (n = 0; n < count; n++)
+	list_add_tail(&(list[n].list),&ng_aconv);
     return 0;
 }
 
@@ -395,7 +422,7 @@ ng_filter_register(int magic, char *plugname, struct ng_filter *filter)
 {
     if (0 != ng_check_magic(magic,plugname,"filter"))
 	return -1;
-    ng_register_listadd((void***)(&ng_filters),filter);
+    list_add_tail(&filter->list,&ng_filters);
     return 0;
 }
 
@@ -404,7 +431,16 @@ ng_writer_register(int magic, char *plugname, struct ng_writer *writer)
 {
     if (0 != ng_check_magic(magic,plugname,"writer"))
 	return -1;
-    ng_register_listadd((void***)(&ng_writers),writer);
+    list_add_tail(&writer->list,&ng_writers);
+    return 0;
+}
+
+int
+ng_reader_register(int magic, char *plugname, struct ng_reader *reader)
+{
+    if (0 != ng_check_magic(magic,plugname,"reader"))
+	return -1;
+    list_add_tail(&reader->list,&ng_readers);
     return 0;
 }
 
@@ -413,7 +449,7 @@ ng_vid_driver_register(int magic, char *plugname, struct ng_vid_driver *driver)
 {
     if (0 != ng_check_magic(magic,plugname,"video drv"))
 	return -1;
-    ng_register_listadd((void***)(&ng_vid_drivers),driver);
+    list_add_tail(&driver->list,&ng_vid_drivers);
     return 0;
 }
 
@@ -422,7 +458,7 @@ ng_dsp_driver_register(int magic, char *plugname, struct ng_dsp_driver *driver)
 {
     if (0 != ng_check_magic(magic,plugname,"dsp drv"))
 	return -1;
-    ng_register_listadd((void***)(&ng_dsp_drivers),driver);
+    list_add_tail(&driver->list,&ng_dsp_drivers);
     return 0;
 }
 
@@ -431,37 +467,87 @@ ng_mix_driver_register(int magic, char *plugname, struct ng_mix_driver *driver)
 {
     if (0 != ng_check_magic(magic,plugname,"mixer drv"))
 	return -1;
-    ng_register_listadd((void***)(&ng_mix_drivers),driver);
+    list_add_tail(&driver->list,&ng_mix_drivers);
     return 0;
 }
 
 struct ng_video_conv*
-ng_conv_find(int out, int *i)
+ng_conv_find_to(unsigned int out, int *i)
 {
-    struct ng_video_conv *ret = NULL;
-    
-    for (; ng_conv[*i] != NULL; (*i)++) {
+    struct list_head *item;
+    struct ng_video_conv *ret;
+    int j = 0;
+
+    list_for_each(item,&ng_conv) {
+	if (j < *i) {
+	    j++;
+	    continue;
+	}
+	ret = list_entry(item, struct ng_video_conv, list);
 #if 0
-	fprintf(stderr,"\tconv:  %-28s =>  %s\n",
-		ng_vfmt_to_desc[ng_conv[*i]->fmtid_in],
-		ng_vfmt_to_desc[ng_conv[*i]->fmtid_out]);
+	fprintf(stderr,"\tconv to:  %-28s =>  %s\n",
+		ng_vfmt_to_desc[ret->fmtid_in],
+		ng_vfmt_to_desc[ret->fmtid_out]);
 #endif
-	if (ng_conv[*i]->fmtid_out == out) {
-	    ret = ng_conv[*i];
+	if (ret->fmtid_out == out) {
 	    (*i)++;
-	    break;
+	    return ret;
+	}
+	(*i)++;
+	j++;
+    }
+    return NULL;
+}
+
+struct ng_video_conv*
+ng_conv_find_from(unsigned int in, int *i)
+{
+    struct list_head *item;
+    struct ng_video_conv *ret;
+    
+    int j = 0;
+
+    list_for_each(item,&ng_conv) {
+	if (j < *i) {
+	    j++;
+	    continue;
+	}
+	ret = list_entry(item, struct ng_video_conv, list);
+#if 0
+	fprintf(stderr,"\tconv from:  %-28s =>  %s\n",
+		ng_vfmt_to_desc[ret->fmtid_in],
+		ng_vfmt_to_desc[ret->fmtid_out]);
+#endif
+	if (ret->fmtid_in == in) {
+	    (*i)++;
+	    return ret;
 	}
     }
-    return ret;
+    return NULL;
+}
+
+struct ng_video_conv*
+ng_conv_find_match(unsigned int in, unsigned int out)
+{
+    struct list_head *item;
+    struct ng_video_conv *ret = NULL;
+    
+    list_for_each(item,&ng_conv) {
+	ret = list_entry(item, struct ng_video_conv, list);
+	if (ret->fmtid_in  == in && ret->fmtid_out == out)
+	    return ret;
+    }
+    return NULL;
 }
 
 /* --------------------------------------------------------------------- */
 
 const struct ng_vid_driver*
-ng_vid_open(char *device, struct ng_video_fmt *screen, void *base,
-	    void **handle)
+ng_vid_open(char *device, char *driver, struct ng_video_fmt *screen,
+	    void *base, void **handle)
 {
-    int i;
+    struct list_head *item;
+    struct ng_vid_driver *drv;
 
 #ifdef __linux__
     if (NULL != screen) {
@@ -480,91 +566,129 @@ ng_vid_open(char *device, struct ng_video_fmt *screen, void *base,
 #endif
 
     /* check all grabber drivers */
-    if (NULL == ng_vid_drivers)
-	return NULL;
-    for (i = 0; NULL != ng_vid_drivers[i]; i++) {
+    list_for_each(item,&ng_vid_drivers) {
+        drv = list_entry(item, struct ng_vid_driver, list);
+	if (driver && 0 != strcasecmp(driver, drv->name))
+	    continue;
 	if (ng_debug)
-	    fprintf(stderr,"vid-open: trying: %s... \n",
-		    ng_vid_drivers[i]->name);
-	if (NULL != (*handle = ng_vid_drivers[i]->open(device)))
+	    fprintf(stderr,"vid-open: trying: %s... \n", drv->name);
+	if (NULL != (*handle = drv->open(device)))
 	    break;
 	if (ng_debug)
-	    fprintf(stderr,"vid-open: failed: %s\n",ng_vid_drivers[i]->name);
+	    fprintf(stderr,"vid-open: failed: %s\n",drv->name);
     }
-    if (NULL == ng_vid_drivers[i])
+    if (item == &ng_vid_drivers)
 	return NULL;
     if (ng_debug)
-	fprintf(stderr,"vid-open: ok: %s\n",ng_vid_drivers[i]->name);
-    if (NULL != screen &&
-	ng_vid_drivers[i]->capabilities(*handle) & CAN_OVERLAY) {
-	ng_vid_drivers[i]->setupfb(*handle,screen,base);
-    }
-    return ng_vid_drivers[i];
+	fprintf(stderr,"vid-open: ok: %s\n",drv->name);
+    if (NULL != screen && drv->capabilities(*handle) & CAN_OVERLAY)
+	drv->setupfb(*handle,screen,base);
+    return drv;
 }
 
 const struct ng_dsp_driver*
-ng_dsp_open(char *device, struct ng_audio_fmt *fmt, void **handle)
+ng_dsp_open(char *device, struct ng_audio_fmt *fmt, int record, void **handle)
 {
-    int i;
-    
+    struct list_head *item;
+    struct ng_dsp_driver *drv;
+
     /* check all dsp drivers */
-    if (NULL == ng_dsp_drivers)
-	return NULL;
-    for (i = 0; NULL != ng_dsp_drivers[i]; i++) {
-	if (NULL == ng_dsp_drivers[i]->name)
+    list_for_each(item,&ng_dsp_drivers) {
+        drv = list_entry(item, struct ng_dsp_driver, list);
+	if (NULL == drv->name)
+	    continue;
+	if (record && NULL == drv->read)
+	    continue;
+	if (!record && NULL == drv->write)
 	    continue;
 	if (ng_debug)
-	    fprintf(stderr,"dsp-open: trying: %s... \n",
-		    ng_dsp_drivers[i]->name);
-	if (NULL != (*handle = ng_dsp_drivers[i]->open(device,fmt)))
+	    fprintf(stderr,"dsp-open: trying: %s... \n", drv->name);
+	if (NULL != (*handle = drv->open(device,fmt,record)))
 	    break;
 	if (ng_debug)
-	    fprintf(stderr,"dsp-open: failed: %s\n",ng_dsp_drivers[i]->name);
+	    fprintf(stderr,"dsp-open: failed: %s\n", drv->name);
     }
-    if (NULL == ng_dsp_drivers[i])
+    if (item == &ng_dsp_drivers)
 	return NULL;
     if (ng_debug)
-	fprintf(stderr,"dsp-open: ok: %s\n",ng_dsp_drivers[i]->name);
-    return ng_dsp_drivers[i];
+	fprintf(stderr,"dsp-open: ok: %s\n",drv->name);
+    return drv;
 }
 
 struct ng_attribute*
 ng_mix_init(char *device, char *channel)
 {
+    struct list_head *item;
+    struct ng_mix_driver *drv = NULL;
     struct ng_attribute *attrs = NULL;
-    int i;
+    void *handle;
     
     /* check all mixer drivers */
-    if (NULL == ng_mix_drivers)
-	return NULL;
-    for (i = 0; NULL != ng_mix_drivers[i]; i++) {
-	if (NULL == ng_mix_drivers[i]->name)
-	    continue;
+    list_for_each(item, &ng_mix_drivers) {
+        drv = list_entry(item, struct ng_mix_driver, list);
 	if (ng_debug)
-	    fprintf(stderr,"mix-init: trying: %s... \n",
-		    ng_mix_drivers[i]->name);
-	if (NULL != (attrs = ng_mix_drivers[i]->init(device,channel)))
-	    break;
+	    fprintf(stderr,"mix-init: trying: %s... \n", drv->name);
+	if (NULL != (handle = drv->open(device))) {
+	    if (NULL != (attrs = drv->volctl(handle,channel)))
+		break;
+	    drv->close(handle);
+	}
 	if (ng_debug)
-	    fprintf(stderr,"mix-init: failed: %s\n",ng_mix_drivers[i]->name);
+	    fprintf(stderr,"mix-init: failed: %s\n",drv->name);
     }
     if (ng_debug && NULL != attrs)
-	fprintf(stderr,"mix-init: ok: %s\n",ng_mix_drivers[i]->name);
+	fprintf(stderr,"mix-init: ok: %s\n",drv->name);
     return attrs;
 }
 
-long long
+struct ng_reader* ng_find_reader(char *filename)
+{
+    struct list_head *item;
+    struct ng_reader *reader;
+    char blk[512];
+    FILE *fp;
+    int m;
+
+    if (NULL == (fp = fopen(filename, "r"))) {
+	fprintf(stderr,"open %s: %s\n",filename,strerror(errno));
+        return NULL;
+    }
+    memset(blk,0,sizeof(blk));
+    fread(blk,1,sizeof(blk),fp);
+    fclose(fp);
+
+    list_for_each(item,&ng_readers) {
+	reader = list_entry(item, struct ng_reader, list);
+	for (m = 0; m < 4 && reader->mlen[m] > 0; m++) {
+	    if (0 == memcmp(blk+reader->moff[m],reader->magic[m],
+			    reader->mlen[m]))
+		return reader;
+	}
+    }
+    if (ng_debug)
+	fprintf(stderr,"%s: no reader found\n",filename);
+    return NULL;
+}
+
+int64_t
+ng_tofday_to_timestamp(struct timeval *tv)
+{
+    long long ts;
+
+    ts  = tv->tv_sec;
+    ts *= 1000000;
+    ts += tv->tv_usec;
+    ts *= 1000;
+    return ts;
+}
+
+int64_t
 ng_get_timestamp()
 {
     struct timeval tv;
-    long long ts;
 
     gettimeofday(&tv,NULL);
-    ts  = tv.tv_sec;
-    ts *= 1000000;
-    ts += tv.tv_usec;
-    ts *= 1000;
-    return ts;
+    return ng_tofday_to_timestamp(&tv);
 }
 
 struct ng_video_buf*
@@ -696,20 +820,19 @@ void ng_check_clipping(int width, int height, int xadjust, int yadjust,
 
 static int ng_plugins(char *dirname)
 {
-    struct dirent *ent;
+    struct dirent **list;
     char filename[1024];
     void *plugin;
     void (*initcall)(void);
-    DIR *dir;
-    int n = 0;
+    int i,n = 0,l = 0;
 
-    dir = opendir(dirname);
-    if (NULL == dir)
-	return n;
-    while (NULL != (ent = readdir(dir))) {
-	if (0 != fnmatch("*.so",ent->d_name,0))
+    n = scandir(dirname,&list,NULL,alphasort);
+    if (n <= 0)
+	return 0;
+    for (i = 0; i < n; i++) {
+	if (0 != fnmatch("*.so",list[i]->d_name,0))
 	    continue;
-	sprintf(filename,"%s/%s",dirname,ent->d_name);
+	sprintf(filename,"%s/%s",dirname,list[i]->d_name);
 	if (NULL == (plugin = dlopen(filename,RTLD_NOW))) {
 	    fprintf(stderr,"dlopen: %s\n",dlerror());
 	    continue;
@@ -721,17 +844,19 @@ static int ng_plugins(char *dirname)
 	    }
 	}
 	initcall();
-	n++;
+	l--;
     }
-    closedir(dir);
-    return n;
+    for (i = 0; i < n; i++)
+	free(list[i]);
+    free(list);
+    return l;
 }
 
 void
 ng_init(void)
 {
     static int once=0;
-    int count;
+    int count=0;
 
     if (once++) {
 	fprintf(stderr,"panic: ng_init called twice\n");
@@ -742,16 +867,12 @@ ng_init(void)
     ng_color_yuv2rgb_init();
     ng_writefile_init();
 
-    count = ng_plugins(LIBDIR);
-    if (0 == count)
+    count += ng_plugins(LIBDIR);
+    if (0 == count) {
 	/* nice for development */
-	count = ng_plugins("../libng/plugins");
+	count += ng_plugins("../libng/plugins");
+	count += ng_plugins("../libng/contrib-plugins");
+    }
     if (0 == count)
 	fprintf(stderr,"WARNING: no plugins found [%s]\n",LIBDIR);
 }
-
-/*
- * Local variables:
- * compile-command: "(cd ..; make)"
- * End:
- */
