@@ -222,7 +222,7 @@ XtResource args_desc[] = {
 	"vidmode",
 	XtCBoolean, XtRBoolean, sizeof(int),
 	XtOffset(struct ARGS*,vidmode),
-	XtRString, "0"
+	XtRString, "1"
     },{
 	"dga",
 	XtCBoolean, XtRBoolean, sizeof(int),
@@ -2124,12 +2124,14 @@ int xt_vm_randr_input_init(Display *dpy)
     if (debug)
 	fprintf(stderr,"xt: checking for randr extention ...\n");
     xfree_randr_init(dpy);
-    if (fs_width && fs_height && !args.vidmode && !have_randr) {
+#if 0
+    if (fs_width && fs_height && !args.vidmode) {
 	if (debug)
 	    fprintf(stderr,"fullscreen mode configured (%dx%d), "
 		    "VidMode extention enabled\n",fs_width,fs_height);
 	args.vidmode = 1;
     }
+#endif
     if (debug)
 	fprintf(stderr,"xt: checking for vidmode extention ...\n");
     xfree_vm_init(dpy);
